<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with BWS REST API.
.DESCRIPTION
    Authenticate and call "Search users" BWS REST API.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    TenantGuid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER paging
    A switch to enable paging of the results returned.
.LINK
    https:// help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
.EXAMPLE
    .\GetUsersDetails.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType AD -username admin -password password -domain example
    .\GetUsersDetails.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password
    .\GetUsersDetails.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -paging
#>

Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Blah"
    )]
    [switch]$paging
)

$global:authString = $null;
$global:uemBaseUrl = $null;
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $global:uemBaseUrl = "https://" + $uemHostAndPort + "/" + $tenantGuid + "/api/v1";

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    # Create body for REST call
    $body = @{}
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));
    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = $global:uemBaseUrl + "/util/authorization";

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        $global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:authString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
    * Prompts user to enter any key to continue to the next page of users.
    *
    * @param message
    *       Pause message to display.
#>
function pause ($message)
{
    # Check if running Powershell ISE
    if ($psISE)
    {
        Add-Type -AssemblyName System.Windows.Forms
        [System.Windows.Forms.MessageBox]::Show("$message")
    }
    else
    {
        Write-Host "$message" -ForegroundColor Yellow
        $x = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
    }
}

<#
    * Retrieves 10 users at a time in pages sorting by username DESC.
#>
function getUsersAndDisplayUserData()
 {
    $methodName = "getUsersAndDisplayUserData()";
    enterMethod($methodName);
    $userList = @{};
    $userList = $null;
    $pageCounter = 0;
    $offset = 0;
    $count = 0;

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    while($offset -le $count)
    {
        $offset = $pageCounter * 10;
        $request = $global:uemBaseUrl + "/users?max=10&offset=" + $offset + "&sortBy=username%20DESC";

        # Invoke RESTMethod to call users API to obtain users in the system
        try
        {
            $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
        }
        catch
        {
             handleError $_;
        }

        if($response -ne $null -AND $response.users.length -gt 0)
        {
            $pageCounter++;
            $count = $response.users.length + $count;
            foreach($user in $response.users)
            {
                displayUserDetails($user);
            }
        }
        else
        {
            Write-Host("No users were found");
            break;
        }
        if($paging)
        {
            pause "`nPress Any Key For Next Page of Users";
        }
    }

    exitMethod($methodName);
 }

<#
    * Display details for a user.
    *
    * @param user
    *       User to display details for.
#>
function displayUserDetails($user)
{
    $methodName = "displayUserDetails()";
    enterMethod($methodName);
    $response = $null;

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.userdetail-v1+json");

    $request = $global:uemBaseUrl  +"/users/" + $user.guid + "/userDevices";

    # Invoke RESTMethod to call userDevices
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
        handleError $_;
    }

    Write-Host("--------------------------");
    Write-Host("User name: " +$user.username);
    Write-host("Display name: "+$user.displayName);
    Write-host("First name: " + $user.firstName);
    Write-host("Last name: " + $user.lastName);
    Write-host("Eamil address: " + $user.emailAddress);
    Write-host("User Guid: " + $user.guid);

    if($user.directoryId -ne $null)
    {
        Write-host("Directory Id: " + $user.directoryId);
    }
    if($response.userDevices.length -gt 0)
        {
            foreach($userDevice in $response.userDevices)
            {
                Write-Host("-- Device Details --");
                Write-Host("Enrollment Type: " + $userDevice.enrollmentType);
                Write-host("Activation Date: " + $userDevice.device.activationDate);
                Write-Host("Hardware Name: " + $userDevice.device.hardwareName);
                Write-Host("Hardware Model: " + $userDevice.device.hardwareModel);
                Write-Host("OS: "+ $userDevice.device.os);
                Write-Host("");
            }
        }
    else
    {
        Write-Host("No devices found for user");
    }
    exitMethod($methodName);
 }

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}
function main
{
    $methodName = "Main()";
    enterMethod($methodName);
    try
    {
        if (setup)
        {
            Write-Host("Setup Complete...");
            getUsersAndDisplayUserData;
        }
    }
    catch
    {
        handleError($_);
    }
    exitMethod($methodName);
    Write-Host("Complete!");
    exit(0);
}

# Call main function.
main;